
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <locale.h>
#include <unistd.h>
#include <sys/stat.h>
#include <ctype.h>
#include <time.h>
#include <fcntl.h>
#include <stdbool.h>
#include <inttypes.h>  // for intmax_t

#include <glib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>

#include <menu-cache.h>

#define LOCK_NAME "popupmenu_lockfile"
#define MAX_PATH_LEN 256

// Global variables
gint weight, height;
int i, numDirs;
GError *error = NULL;

typedef struct _Items {
	pthread_t thread;
	
	// Backend items
	MenuCacheDir *dir;
	
	// Frontend items     	
	GtkWidget    *widget;  
	GtkMenuItem  *menu_item;
} Items, *ptrItems;

struct dirAttr {
	const gchar *name;
	GtkWidget   *image;
};

struct appAttr {
	const gchar *name;
	const gchar *exec;
	GtkWidget   *image;
};

struct sbuf {
	char *buf;
	int bufsiz;
	int len;
};

// Function prototypes
void menu_position (GtkMenu*, gint*, gint*, gboolean*, gpointer); 
void display_children (void*);
void run_thread (void*);
struct dirAttr get_dir_attr (MenuCacheDir*);
void display_header (const gchar*, GtkWidget*);
void display_dir_widget (MenuCacheDir*, GtkWidget*, GtkWidget*);
MenuCacheApp* find_menu_cache_app (MenuCacheDir *dir, const gchar* exec);
struct appAttr get_app_attr (MenuCacheApp*);
void display_app_widget (MenuCacheApp*, GtkWidget*, gboolean);
gchar* get_item_icon_path (MenuCacheItem*);
gchar* set_cmd (const gchar*);
static void run_cmd (const gchar*);
static void quit_menu ();
bool file_exists (const gchar*);
void sbuf_init (struct sbuf*);
void sbuf_addstr(struct sbuf*, const char*);
void sbuf_concat(struct sbuf*, int, ...);
int already_running (void);
void find_missing_icon (struct sbuf*, const gchar*, int count, ...);
int delete_dir(const char*);
void get_tempdir(struct sbuf*);

int main (int argc, char **argv)
{
	bool bSetPos=FALSE;
	// Parse args:
	switch(argc) {
		case 2: bSetPos=TRUE;
				break;
		default: bSetPos=FALSE;
				 break;
	}
	
	Items root;
	MenuCache *menu_cache = NULL;
	DIR *dirp = NULL;
	struct dirent *ent = NULL;
	struct dirent *tmp_ent = NULL;

    // Some hot keys (keybindings) get carried away and start
    // many instances. Will try to use a lock file so that
    // only one instance of this program runs at a given time.
    // If this check causes you problems, comment it out.
    int i_already_running = already_running();
	if (i_already_running == 1) {
        fprintf(stderr, "All ready running, will quit.\n");
        return EXIT_FAILURE;
    }
	else if (i_already_running == 2) {
        fprintf(stderr, "%s: Error in routine already_running(), "
                        "will quit.\n", argv[0]);
        return EXIT_FAILURE;
    }

	if (!gtk_init_check (&argc, &argv)) {
		g_print("Error, cannot initialize gtk.\n");
		exit (EXIT_FAILURE);
	}

	// Global system variables
	setlocale (LC_ALL, "");
	g_unsetenv("XDG_MENU_PREFIX");

	gtk_init (&argc, &argv);
	
	if (!gtk_icon_size_lookup (GTK_ICON_SIZE_BUTTON, &weight, &height)) { 
		weight = 24;
		height = 24;
	};
	
	menu_cache = menu_cache_lookup_sync ("ob-applications.menu");
	if (!menu_cache )
	{
		g_warning ("\nCannot connect to menu-cache\n");
		exit(-1);
	}	
	
	// BACKEND's root_dir (the first item in the hierarchy is "Aplications")
	root.dir = menu_cache_dup_root_dir (menu_cache);
	
	// FRONTENS's root_menu
	root.widget = gtk_menu_new ();	

	// Adding the header (static) to the menu	
    GtkWidget* _execute = gtk_image_menu_item_new_from_stock(GTK_STOCK_EXECUTE, NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(root.widget), _execute);
	g_signal_connect_swapped (_execute, "activate", G_CALLBACK (run_cmd), "gmrun");
	
	root.menu_item = gtk_separator_menu_item_new ();
	gtk_menu_shell_append (GTK_MENU_SHELL (root.widget), root.menu_item);

/*	
    gchar *header=g_strconcat(getenv("HOME"), "/.config/popupmenu/header/", NULL);
    dirp = opendir(header);
    while ((ent = readdir(dirp))) {
		if(g_strcmp0(ent->d_name, ".") && g_strcmp0(ent->d_name, "..")) {
			gchar *path=g_strconcat(header, ent->d_name, NULL);
			display_header(path, root.widget);
			g_free(path);
		}
	}
	    
	g_free(header);
	closedir(dirp);
	* */
	
	FILE *f;
    gchar *_cmd=g_strconcat("ls -l ", getenv("HOME"), "/.config/popupmenu/header | cut -d' ' -f9", NULL);
    f = popen (_cmd, "r");
    if(!f) {
		printf("Cannot read file: %s\n", _cmd);
		exit(EXIT_FAILURE);
	}
	
	gchar *str = malloc(sizeof(gchar) * 1024);
	if (!str) {
		g_print ("Memory assignment failure.\n");
		exit (EXIT_FAILURE);
	}
	
	fgets( str, 1024, f );			
	while( fgets( str, 1024, f ) ) {
		str[strlen(str)-1] = '\0';
		gchar *path=g_strconcat(getenv("HOME"), "/.config/popupmenu/header/", str, NULL);
		display_header(path, root.widget);
		g_free(path);
	}
	
	pclose(f);
	free(str);
	
	root.menu_item = gtk_separator_menu_item_new ();
	gtk_menu_shell_append (GTK_MENU_SHELL (root.widget), root.menu_item);

	// BACKEND's list ("Accesories", "Education", etc... and separators)
	GSList *children = menu_cache_dir_get_children (root.dir);

	// Adding the dinamic part, only the parent items of the GSList
	// The children will be added in a separate thread,
	// improving the response of the popupmenu
	numDirs=0;
	for ( ; children; children=children->next) 		
		switch ((gint) menu_cache_item_get_type (MENU_CACHE_ITEM(children->data)))
		{
			case MENU_CACHE_TYPE_DIR:
				root.menu_item = gtk_menu_new();
				display_dir_widget (children->data,
				                    root.widget,
									root.menu_item);
				numDirs++;
				break;

			case MENU_CACHE_TYPE_SEP:				
				root.menu_item = gtk_separator_menu_item_new ();
				gtk_menu_shell_append (GTK_MENU_SHELL (root.widget),
				                       root.menu_item);
				break;
		}
		
	// Adding the footer (static) to the menu
	root.menu_item = gtk_separator_menu_item_new ();
	gtk_menu_shell_append (GTK_MENU_SHELL (root.widget), root.menu_item);	
    GtkWidget* _quit = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(root.widget), _quit);
	g_signal_connect_swapped (_quit, "activate", G_CALLBACK (run_cmd), "sh -c \"SUDO_ASKPASS=/usr/bin/gsshaskpass sudo -A /usr/local/bin/shutdown\"");
	
	// Show the menu
	gtk_widget_show_all (root.widget);
	
	// Adding the rest of the items to the menu in a separate thread
	pthread_create(&root.thread, NULL , display_children , &root);
	
	// Signal handlers
	g_signal_connect_swapped (root.widget, "deactivate", G_CALLBACK (quit_menu), NULL);
	
	while(!gtk_widget_get_visible(root.widget)) {
		usleep(50000);
		if(bSetPos)
			gtk_menu_popup (GTK_MENU (root.widget), NULL, NULL, 
				(GtkMenuPositionFunc) menu_position, NULL, 0,
				gtk_get_current_event_time ());
		else
			gtk_menu_popup (GTK_MENU (root.widget), NULL, NULL, 
				NULL, NULL, 0,
				gtk_get_current_event_time ());
		gtk_main_iteration();
	}

	gtk_main ();
	
	if(file_exists(menu_cache->cache_file)) remove(menu_cache->cache_file);		
	menu_cache_unref (menu_cache);
	
	return EXIT_SUCCESS;
}

void menu_position (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer data)
{
	FILE *fp;
    gchar *data_file=g_strconcat(getenv("HOME"), "/.tint2/", "session.0", NULL);
    fp = fopen (data_file, "r");
    if(!fp) {
		printf("Cannot open file %s\n", data_file);
		exit(EXIT_FAILURE);
	}
	g_free(data_file);
		
	gchar *l = malloc (sizeof(gchar) * 8);
	*x = atoi(fgets (l, 8, fp));
	*y = atoi(fgets (l, 8, fp)) +  30;
	*push_in = TRUE;
	
	fclose(fp);
	g_free(l);
}

void display_children (void *pVoid)
{
	ptrItems items = (ptrItems)pVoid;
	Items *params;
	struct sbuf tempdir;
	
	params = malloc(sizeof(Items) * numDirs);
	if (!params) {
		g_print ("Memory assignment failure.\n");
		exit (EXIT_FAILURE);
	}
	
	sbuf_init(&tempdir);
	get_tempdir(&tempdir);
	g_mkdir(tempdir.buf, 0700);
	
	struct dirent *ent;
	GString *str = NULL;
	
	// BACKEND LIST (children)
	GSList *l = menu_cache_dir_get_children (items->dir);
	
	// FRONTEND LIST (child widgets)
	GList *m = gtk_container_get_children(GTK_CONTAINER(items->widget));
	
	// Skip the header
	while(!GTK_IS_WIDGET(gtk_menu_item_get_submenu (m->data)))	m=g_list_next(m); 

	i=0;
	for ( ; l; l=l->next) 		
		switch ((gint) menu_cache_item_get_type (MENU_CACHE_ITEM(l->data)))
		{
			case MENU_CACHE_TYPE_DIR:
				params[i].dir = l->data;
				params[i].widget = gtk_menu_item_get_submenu (m->data);				
				params[i].menu_item = m->data;
				m=g_list_next(m);
				i++;
				break;

			case MENU_CACHE_TYPE_SEP:
				m=g_list_next(m);				
				break;
		}
	
	for(i=0; i<numDirs; i++) {
		pthread_create (&params[i].thread, NULL , run_thread, &params[i]);
		pthread_join (params[i].thread, NULL);
	}
	
	g_free(params);
	
	//delete_dir(tempdir.buf);
	free(tempdir.buf);
}

void run_thread (void *pVoid)
{
	ptrItems items = (ptrItems)pVoid;
	
	const gchar *widget_label = gtk_menu_item_get_label (items->menu_item);
	// printf("\nDIR = %s\n", widget_label);
	
	// BACKEND LIST (items of the subdir)
	GSList *app = menu_cache_dir_get_children (items->dir);
	
	
	for ( ; app; app=app->next) { 
		const gchar *app_name = menu_cache_item_get_name (MENU_CACHE_ITEM(app->data));	
		// printf("  -----> App = %s\n", app_name);
		display_app_widget (app->data, items->widget, FALSE);
	}
}

struct dirAttr get_dir_attr (MenuCacheDir *dir)
{
	struct dirAttr dir_attr;
	
	const gchar *name = menu_cache_item_get_name (MENU_CACHE_ITEM(dir));	
	
	gchar *icon = get_item_icon_path (MENU_CACHE_ITEM(dir));
	GdkPixbuf *Pixbuf = gdk_pixbuf_new_from_file_at_size (icon, weight, height, &error);
	if (Pixbuf == NULL) {
		g_print ("%s\n", error->message);
		g_error_free (error);
		error = NULL;
	}
		
	GtkWidget *image = gtk_image_new_from_pixbuf (Pixbuf);
	
	dir_attr.name = name;
	dir_attr.image = image;
	
	g_free (icon);
	
	return dir_attr;
}

void display_dir_widget (MenuCacheDir* dir, GtkWidget* widget, GtkWidget* child_widget)
{	
	GtkWidget *menu_item = NULL;
	
	struct dirAttr dir_attr = get_dir_attr(dir);

	// Start the menu_item
	menu_item = gtk_image_menu_item_new_with_mnemonic (dir_attr.name);

	// Add the menu_item to the widget
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(menu_item), dir_attr.image);
	gtk_menu_shell_append (GTK_MENU_SHELL(widget), menu_item);
	
	// Set the menu_item to submenu
	gtk_menu_item_set_submenu (GTK_MENU_ITEM(menu_item), child_widget);

	gtk_widget_show_all (widget);
}

MenuCacheApp* find_menu_cache_app (MenuCacheDir *dir, const gchar* exec) {
	
	GSList *l = menu_cache_dir_get_children (dir);
	
	for ( ; l; l=l->next) {
			
		GSList *m = menu_cache_dir_get_children (l->data);
		
		for ( ; m; m=m->next){
		
			// Skip dirs and separators:
			while((gint) menu_cache_item_get_type(MENU_CACHE_ITEM(m->data)) != MENU_CACHE_TYPE_APP)
				m=m->next;
		
			if(!g_strcasecmp(menu_cache_app_get_exec (MENU_CACHE_ITEM(m->data)), exec))
				return m->data;
				else continue;	
		}
	}
	
	return NULL;
}

struct appAttr get_app_attr (MenuCacheApp *app)
{
	struct appAttr app_attr;
	
	const gchar *name = menu_cache_item_get_name (MENU_CACHE_ITEM(app));
	const gchar *exec = menu_cache_app_get_exec (MENU_CACHE_APP(app));
	gchar *icon = get_item_icon_path (MENU_CACHE_ITEM(app));
	app_attr.name = name;
	
	if (file_exists(icon)) {
		GdkPixbuf *Pixbuf = gdk_pixbuf_new_from_file_at_size (icon, weight, height, &error);
		if (Pixbuf == NULL) {
			g_print ("%s\n", error->message);
			g_error_free (error);
			error = NULL;
		}
		
		GtkWidget *image = gtk_image_new_from_pixbuf (Pixbuf);
		app_attr.image = image;
	}
	
	else app_attr.image = NULL;
	
	app_attr.exec = exec;	
	
	g_free (icon);
	return app_attr;
}

void display_app_widget (MenuCacheApp *app, GtkWidget* widget, gboolean skip_blacklist)
{
	GtkWidget *menu_item = NULL;
	bool skip=FALSE;
	
	const gchar *path = menu_cache_item_get_file_path (MENU_CACHE_ITEM(app));
	
	FILE *fp;
	fp = fopen (path, "r");
	if (!fp) {
		g_print ("Cannot open the file %s.\n", path);
		exit (EXIT_FAILURE);
	}
	
	gchar *str = malloc(sizeof(gchar) * 1024);
	if (!str) {
		g_print ("Memory assignment failure.\n");
		exit (EXIT_FAILURE);
	}
				
	while( fgets( str, 1024, fp ) ) {
		gchar *token;
		const gchar delim[2] = "=";
		token=strtok(str, delim);
		if(!g_strcmp0(token, "OnlyShowIn")) {
			skip=TRUE;
			break;
		}
	}
	
	fclose(fp);
	free(str);
	
	if(skip) return;
	
	struct appAttr app_attr = get_app_attr(app);		
	const gchar *exec = app_attr.exec;

	// Start the menu_item		
	menu_item = gtk_image_menu_item_new_with_mnemonic (app_attr.name);

	// Start the menu_item
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), app_attr.image);	
	gtk_menu_shell_append (GTK_MENU_SHELL (widget), menu_item);	

	// Signal handler	
	g_signal_connect_swapped (menu_item, "activate", G_CALLBACK (run_cmd), exec);

	gtk_widget_show_all (widget);
}

gchar* get_item_icon_path (MenuCacheItem *item)
{
	GtkIconInfo *icon_info = NULL;
	gchar *icon = NULL;
	gchar *tmp_name = NULL;
	GtkIconTheme *icon_theme;
	
	icon_theme = gtk_icon_theme_get_default ();
	
	const gchar *name = menu_cache_item_get_icon (MENU_CACHE_ITEM(item));

	if (name)
	{
		if (g_path_is_absolute (name))
			return g_strdup (name);

		tmp_name = strndup (name, strrchr (name, '.') - name);
		icon_info = gtk_icon_theme_lookup_icon (icon_theme, tmp_name, 24, GTK_ICON_LOOKUP_NO_SVG |
		                                                                  GTK_ICON_LOOKUP_GENERIC_FALLBACK);
		g_free (tmp_name);
		
		if (!g_path_is_absolute(name) && !icon_info) {
			struct sbuf u, v, w, s;
			sbuf_init(&u);
			sbuf_init(&v);
			sbuf_init(&w);
			sbuf_init(&s);
		
			sbuf_concat(&u, 2, name, ".png");
			sbuf_concat(&v, 2, name, ".xpm");
			sbuf_concat(&w, 2, name, ".svg");
			
			find_missing_icon (&s, "/usr/share/icons/hicolor/24x24/apps", 4, name, u.buf, v.buf, w.buf);
			if(s.len>0) icon=g_strdup(s.buf);
			else {
				find_missing_icon (&s, "/usr/share/icons/hicolor/scalable/apps", 2, name, w.buf);
				if(s.len>0) {
					icon=g_strdup(s.buf);
				}
			}
		
			free(u.buf);
			free(v.buf);
			free(w.buf);
			free(s.buf);
		}
	}
	
	if (!icon_info)
		icon_info = gtk_icon_theme_lookup_icon (icon_theme, "empty", 24, GTK_ICON_LOOKUP_NO_SVG);

	if(!icon)
		icon = g_strdup (gtk_icon_info_get_filename (icon_info));
	
	gtk_icon_info_free (icon_info);
	return icon;
}

gchar* set_cmd (const gchar *exec)
{
	char *token, *u, *v;
	const char delim[2] = " ";
	gchar *filepath = NULL;
	
	MenuCacheApp *app;
	
	u = malloc(64 * sizeof(char));  
	if(!u) {
			printf("Memory failure\n");
			exit( EXIT_FAILURE );
	}
	
	v = malloc(64 * sizeof(char));  
	if(!v) {
			printf("Memory failure\n");
			exit( EXIT_FAILURE );
	}
	
	g_return_val_if_fail(exec,"");

	GString *cmd = g_string_sized_new (64);
			
	for (;*exec; ++exec)
	{
		if (*exec == '%')
		{
			++exec;
			switch (*exec)
			{
				case 'u':
				case 'U':
				case 'f':
				case 'F': break;
				case 'd':
				case 'D':
				case 'm':
				case 'n':
				case 'N':
				case 'v': break;
				default:
					g_string_append_c (cmd, '%');
					g_string_append_c (cmd, *exec);
					break;
			}
		}
		else
			g_string_append_c (cmd, *exec);
	}
	
	v = g_strchomp (g_string_free (cmd, FALSE));
	strcpy (u, v);
	token  = strtok (v, delim);
/*	
	if ( !strcmp (token, "gksu" )       ||
	     !strcmp (token, "gksudo" )     ||
	     !strcmp (token, "su-to-root" ) 
	) {
		FILE *fp;
		char *w;
				
		fp = popen ("/usr/lib/popupmenu/bin/read-shadow", "r");
		if(!fp) {
			printf("Cannot open /etc/shadow.\n");
			exit( EXIT_FAILURE );
		}
		
		w = malloc(64 * sizeof(char));  
		if(!w) {
			printf("Memory failure\n");
			exit( EXIT_FAILURE );
		}
		
		int len = strlen(u);
		int m=0, n=len;
		while(n>m){
			char temp = u[m];
			u[m++] = u[--n];
			u[n] = temp;
		}
		
		u  = strtok (u, delim);
		len = strlen(u);
		m=0, n=len;
		while(n>m){
			char temp = u[m];
			u[m++] = u[--n];
			u[n] = temp;
		}
		
		fgets( w, 64, fp );
		w = strcat (w, " ");
		GString *strText = g_string_new (strcat (w, u));
		free (u);
		free (v);
		free (w);
		pclose (fp);
		//printf("%s\n", strText->str);
		return g_strchomp (g_string_free (strText, FALSE));		
	}
*/				
	GString *strText = g_string_new (u);
	free (u);
	free (v);
	//printf("%s\n", strText->str);
	return g_strchomp (g_string_free (strText, FALSE));
}

static void run_cmd (const gchar *exec) {

	gchar *app_cmd = set_cmd (exec);
	
	if (!app_cmd) return;

	if (!g_spawn_command_line_async(app_cmd, &error)) {
		g_print ("Error running command.\n");
		g_print ("%s\n", error->message);
		g_error_free (error);
	}

	g_free (app_cmd);
	
	gtk_main_quit ();
}

static void quit_menu () {
	
	gtk_main_quit ();
}

static void execute () {

	if (!g_spawn_command_line_async("gmrun", &error)) {
		g_print ("Error running command.\n");
		g_print ("%s\n", error->message);
		g_error_free (error);
	}
	
	gtk_main_quit ();
}

static void shutdown () {

	if (!g_spawn_command_line_async("xterm -e sudo /usr/local/bin/shutdown", &error)) {
		g_print ("Error running command.\n");
		g_print ("%s\n", error->message);
		g_error_free (error);
	}
	
	gtk_main_quit ();
}

bool file_exists (const gchar *filename) {
  struct stat buffer;   
  return (stat (filename, &buffer) == 0);
}

void sbuf_init(struct sbuf *s)
{
	s->buf = (char*)malloc(1);
	s->buf[0] = '\0';
	s->bufsiz = 1;
	s->len = 0;
}

void sbuf_addstr(struct sbuf *s, const char *data)
{
	int len;

	if (!data)
		return;
	len = strlen(data);
	if (s->bufsiz <= s->len + len + 1) {
		s->bufsiz = s->bufsiz + len;
		s->buf = realloc(s->buf, s->bufsiz);
	}
	memcpy(s->buf + s->len, data, len);
	s->len += len;
	s->buf[s->len] = 0;
}

void sbuf_concat(struct sbuf *s, int count, ...)
{
    va_list items;
	
    va_start(items, count);  /* Requires the last fixed parameter (to get the address) */
    
    while(count--)
		sbuf_addstr(s, va_arg(items, const char*));
	
    va_end(items);
}

void display_header (const gchar *filename, GtkWidget *widget)
{
	GtkWidget *menu_item = NULL;
	
	gchar *icon_path = NULL;
	
	gchar *name = malloc (sizeof(gchar) * 1024);
	gchar *exec = malloc (sizeof(gchar) * 1024);
	gchar *icon = malloc (sizeof(gchar) * 1024);

	struct sbuf name_entry;
	sbuf_init(&name_entry);

	// Global system variables
	const gchar *_lang = g_getenv("LANG");
	setlocale (LC_ALL, "");
	g_unsetenv("XDG_MENU_PREFIX");

	const char delim[] = "_";
	char *token = strtok(_lang, delim);
	if (strcmp(token, "en")) sbuf_concat(&name_entry, 4, "Name", "[", token, "]");
	else sbuf_addstr(&name_entry, "Name");

	FILE *fp;	
	gchar *l = malloc (sizeof(gchar) * 1024);
	
	if (!l) {
		g_print ("Memory assignment failure.\n");
		exit (EXIT_FAILURE);
	}
		
	fp = fopen (filename, "r");
	if (fp==NULL) {
		g_print ("Cannot open the file %s\n", filename);
		exit (EXIT_FAILURE);
	}
	
	fgets (l, 1024, fp);
	while( fgets (l, 1024, fp) ) {
		int i=0;
		while (l[i]) {
			if (l[i]=='\n') l[i]='\0';
			i++;
		}
		
		const char delim[] = "=";
		char *token = strtok(l, delim);
		
			if (!g_strcmp0(name_entry.buf, token)) {
				token = strtok(NULL, delim);
				g_stpcpy(name, token);
			}
			else if (!g_strcmp0("Exec", token)) {
				token = strtok(NULL, delim);
				g_stpcpy(exec, token);
			}
			else if (!g_strcmp0("Icon", token)) {
				token = strtok(NULL, delim);
				g_stpcpy(icon, token);
			}
	}
	
	GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();

	if (icon)
	{
		if (g_path_is_absolute (icon)) icon_path = g_strdup (icon);
		else {
			GtkIconInfo *icon_info = NULL;
			gchar *tmp_name = strndup (icon, strrchr (icon, '.') - icon);
			icon_info = gtk_icon_theme_lookup_icon (icon_theme, tmp_name, 24, GTK_ICON_LOOKUP_NO_SVG |
		                                                                  GTK_ICON_LOOKUP_GENERIC_FALLBACK);
			if (!icon_info)
				icon_info = gtk_icon_theme_lookup_icon (icon_theme, "empty", 24, GTK_ICON_LOOKUP_NO_SVG);
			icon_path = g_strdup (gtk_icon_info_get_filename (icon_info));
			g_free (tmp_name);
			gtk_icon_info_free (icon_info);
		}
	}
	
	GtkWidget *image=NULL;
	GdkPixbuf *Pixbuf = gdk_pixbuf_new_from_file_at_size (icon_path, weight, height, &error);
	if (Pixbuf == NULL) {
		g_print ("%s\n", error->message);
		g_error_free (error);
		error = NULL;
	}
		
	image = gtk_image_new_from_pixbuf (Pixbuf);

	// Start the menu_item		
	menu_item = gtk_image_menu_item_new_with_mnemonic (name);
	
	// Start the menu_item
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), image);
	gtk_menu_shell_append (GTK_MENU_SHELL (widget), menu_item);	
	
	// Signal handler
	g_signal_connect_swapped (menu_item, "activate", G_CALLBACK (run_cmd), exec);

	gtk_widget_show_all (widget);
		
	free (l);
	fclose (fp);
		
	free(name_entry.buf);

	g_free(name);
	g_free(icon);
	g_free(icon_path);
}

int already_running (void)
{
	struct flock fl;
	int fd;
    char *home;
    int n;
    bool ret;
    char *Lock_path;

    // Use lock file 'LOCK_NAME' to see if program is already running.
    // 0 = No, 1 = Yes, 2 = Error

    // Place lock in the home directory of user
    // If user messes with "HOME", will probably have problems
    home = (char *) getenv("HOME");

	Lock_path = malloc(MAX_PATH_LEN + 1);
    n = snprintf(Lock_path, MAX_PATH_LEN, "%s/.%s", home, LOCK_NAME);

	if (n > MAX_PATH_LEN || Lock_path == NULL) {
        fprintf(stderr, "Error, path name too long: %s.\n", Lock_path);
        ret = 2;
        goto Done;
     }

	fd = open(Lock_path, O_RDWR | O_CREAT, 0600);
	if (fd < 0) {
        fprintf(stderr, "Error opening %s.\n", Lock_path);
        ret = 2;
        goto Done;
     }

	fl.l_start = 0;
	fl.l_len = 0;
	fl.l_type = F_WRLCK;
	fl.l_whence = SEEK_SET;

	if (fcntl(fd, F_SETLK, &fl) < 0)
        ret = true;
    else
        ret = false;

Done:
    free(Lock_path);

    return ret;
}

/**
 * \brief Variadic function
 */

void find_missing_icon (struct sbuf *s, const gchar *path, int count, ...)
{
	va_list items;
	struct dirent *ent;
	GString *str = NULL;
	
    DIR *dirp = opendir (path);
    while(ent = readdir(dirp))
	{
		int i=0;
		va_start(items, count);  // Requires the last fixed parameter (to get the address)
		while (i<count) {
			if((fnmatch(va_arg(items, const gchar*), ent->d_name,0)) == 0) {
				sbuf_concat(s, 3, path, "/", ent->d_name);
				break;
			}
			
			i++;
		}
		
		if(s->len>0) break;		
	}
	
	va_end(items);
	closedir(dirp);
}

int delete_dir(const gchar *path)  
{
	const gchar* p;
	GDir* dr;
	GError* gerr;   
	int res=-1;
	
	dr = g_dir_open(path, 0, &gerr);
	if (dr) {
		res = 0;
		while (!res && (p=g_dir_read_name(dr))) {
			if (!g_strcmp0(p, ".") || !g_strcmp0(p, "..")) continue;				
			struct sbuf path_i;			
			sbuf_init(&path_i);			
			sbuf_concat(&path_i, 3, path, "/", p);
			
			if (g_file_test(path_i.buf, G_FILE_TEST_IS_DIR) != 0)
				res=delete_dir(path_i.buf);   /* remove content recursively */
			else
				res=g_unlink(path_i.buf);
					
			free(path_i.buf);
		}	
	}
		
	g_dir_close(dr);
	if (res == 0) res=g_rmdir(path);	
	return res;
}

void get_tempdir(struct sbuf *dirname)
{	
	pid_t pid;	
	pid=getpid();
	char *mypid = NULL;
	if (asprintf(&mypid, "%jd", (intmax_t) pid) != -1) {
		sbuf_concat(dirname, 3, "/tmp/popupmenu-", mypid, ".tmp");
		free(mypid);
	}
}
	
	


